//game loop state defines
#define INIT_STATE		0
#define MENU_STATE		1
#define CONSOLE_STATE	2
#define LOADING_STATE	3
#define RUN_STATE		4
#define EXIT_STATE		5

//game globals
int current_game_state = INIT_STATE; //the initial game state

void InitGame(){}

void DisplayMenu(){}

void DisplayConsole(){}

void DoFinalSetup(){}

void InputHandling(){}

void InitPhysics(){}

void InitGraphicsSystem(){}

void InitAI(){}

void Cleanup(){}

//the game loop will generally be implemented here
int main()
{
	//the game event loop
	while(current_game_state != EXIT_STATE)
	{
		//switch on the game's state
		switch(current_game_state)
		{
			case INIT_STATE:
			{
				// allocate memory, resources, etc:
				InitGame();
			}
			break;

			case MENU_STATE:
			{
				//call and display the game menu using some function, for example:
				DisplayMenu();
			}
			break;

			case CONSOLE_STATE:
			{
				//call and display the game's debug console using some function, for example:
				DisplayConsole();
			}
			break;

			case LOADING_STATE:
			{
				//do the final initialisation before the gameplay starts, for example:
				DoFinalSetup();

				//once this is done, switch to the run state
				current_game_state = RUN_STATE;
			}
			break;

			case RUN_STATE:
			{
				//execute all the game logic and game subsystems, for example:
				InputHandling();
				InitPhysics();
				InitGraphicsSystem();
				InitAI();
				//...

				//the run state can only be terminated through user or system intervention
			}
			break;

			case EXIT_STATE:
			{
				//release all devices, resources and do final cleanup
				Cleanup();

				//once this state is reached the program will fall out of the main while loop
				//during the following iteration.

			}
			break;

			default: break;

		}
	}
}

